#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
# @Version     : Python 3.11.4
# @Software    : Sublime Text 4
# @Author      : StudentCWZ
# @Email       : StudentCWZ@outlook.com
# @Date        : 2023/11/11 14:50
# @File        : init_apscheduler.py
# @Description :
"""

from apscheduler.triggers.interval import IntervalTrigger
from flask import Flask

from application.libs.tasks import task
from application.libs.flask_apscheduler import APScheduler


scheduler = APScheduler()


def init_tasks(app: Flask) -> None:
    """
    Initialize the task extension.

    :param app: The Flask application instance.
    :return: None
    """
    if not isinstance(app, Flask):
        raise TypeError(f'Expected a Flask application instance, got {type(app).__name__}')

    scheduler.api_enabled = True
    scheduler.init_app(app)

    # Create an interval trigger
    interval = IntervalTrigger(
        days=1,
        start_date=app.config.Scheduler.Start,
        end_date=app.config.Scheduler.End,
        timezone=app.config.Scheduler.Timezone)

    # Add a job to the scheduler
    scheduler.add_job(func=task, trigger=interval, id='task_one')

    # Start the scheduler
    scheduler.start()
